# To simulate the status
import random

# Tell if we are working in the Raspberry or if we are working with our fake
# interface
isReal = False

## Setup the GPIO module
def start():
    if isReal:
        GPIO.setmode(GPIO.BCM)

## Clean the GPIO channel so that it can be used by other programs
def stop():
    if isReal:
        GPIO.cleanup()

## Sets some pins as output pins
def setOutput(pin):
    if isReal:
        GPIO.setup(pin, GPIO.OUT)
        GPIO.output(pin, GPIO.HIGH)

## Toggle the status of a pin
def toggle(pin):
    new = not getStatus(pin)
    setStatus(pin, new)
    return new

# NOTE: all my functions are in inverted logic because the relay board I use
# uses high level for off,

## Get the status of the pin
def getStatusReal(pin):
    return not bool(GPIO.input(pin))

## Set the status of a pin
def setStatusReal(pin, status):
    GPIO.output(pin, GPIO.LOW if status else GPIO.HIGH)

## A fake getStatus function to test on a non-Raspberry machine the bot.
# It tells randomly the staus of a lamp.
def getStatusFake(pin):
    print('Fake status of pin ' + str(pin))
    return bool(random.getrandbits(1))

## A fake setStatus function to test on a non-Raspberry machine the bot
def setStatusFake(pin, status):
    print('Fake setting ' + str(pin) + ' as ' + str(status))

# To tell rasperry from other machines, try to import the GPIO module.
# Note that we set aliases just for getStatus and setStatus callbacks because
# they are called several times, whereas other functions are called once,
# therefore just checking if the isReal variable is true makes sense
try:
    import RPi.GPIO as GPIO
    isReal = True

    getStatus = getStatusReal
    setStatus = setStatusReal
except:
    getStatus = getStatusFake
    setStatus = setStatusFake
