import pickle
import os.path
from time import time

from telegram import User

## Manage permissions of the bot
class Permissions:

    ## The super administrator, that cannot be removed
    #
    # The format is (telegram ID, name of the user)
    SUPERADMIN = (your-telegram-id, 'Your Name')

    ## Keep unknown users for one hour to add them if necessary
    KEEP_UNKNOWN = 3600

    ## The file in which we keep the permissions
    FILE = os.path.abspath(os.path.dirname(__file__) + '/permissions.pickle')

    ## Initializes the user dictionary
    def __init__(self):
        self.load()
        self.users[self.SUPERADMIN[0]] = (self.SUPERADMIN[1], None)

        # Map user => last time seen
        self.unknown = {}

    ## Load permissions from a file
    def load(self):
        self.users = None
        try:
            with open(self.FILE, 'rb') as f:
                self.users = pickle.load(f)
        except:
            pass
        finally:
            if not isinstance(self.users, dict):
                self.users = {}

    ## Save permissions to a file
    def save(self):
        try:
            with open(self.FILE, 'wb') as f:
                pickle.dump(self.users, f)
        except:
            pass

    ## Tells if a certain Telegram user is known by the bot
    def isKnown(self, user):
        if user.id in self.users:
            return True
        elif user:
            # Store unknown users to list them and add it using the useradmin
            # conversation
            self.unknown[user] = time()

        return False

    ## Tells if a user is an administrator
    def isAdmin(self, user):
        return user.id == self.SUPERADMIN[0]

    ## Tells if a user is the superadministrator
    def isSuperAdmin(self, user):
        if isinstance(user, User):
            return user.id == self.SUPERADMIN[0]
        elif isinstance(user, int):
            return user == self.SUPERADMIN[0]
        else:
            return False

    ## Return the list of users (only names)
    def getUserList(self):
        l = []
        for (id_, user) in self.users.items():
            l.append((id_, user[0]))
        return l

    ## Return the list of users that tried to reach the bot
    def getUnknown(self):
        now = time()
        self.unknown = {user: t for user, t in self.unknown.items() \
                if now - t < self.KEEP_UNKNOWN}

        return list(self.unknown.keys())

    ## Add a user
    def addUser(self, user, name):
        self.users[user.id] = (name, None)
        self.save()

    ## Delete a user
    def deleteUser(self, userId):
        if userId not in self.users:
            return

        del self.users[userId]
        self.save()
