import random
import re

from telegram import ReplyKeyboardMarkup
from telegram.ext import (ConversationHandler, CommandHandler, RegexHandler,
        MessageHandler, Filters)

from utilities import *

## The handler for the administration of users
class UserAdmin(ConversationHandler, SharedTexts):

    USER_CURRENT = 'Per il momento posso servire:'

    # User adding sentences
    USER_ADD = em('Aggiungi utente :heavy_plus_sign:')
    USER_WHO = 'Chi vuoi aggiungere?'
    USER_ADD_NAME = 'Come si chiama questa persona?'
    USER_NO_UNKNOWN = em('Nessuno sconosciuto ha tentato di usarmi di ' + \
            'recente :thinking_face: \nTi riporto al menù principale...')
    USER_INVALID_NAME = em(':warning: Il nome non può cominciare con / .' + \
            '\nPer favore inseriscine un altro.')
    WELCOME_NEW = em('Lieto di servirti! :nerd_face:')

    # User removing sentences
    USER_RM = em('Rimuovi utente :heavy_minus_sign:')
    USER_RM_QUERY = 'Ok, inserisci il nome dell\'utente che vuoi rimuovere'
    USER_RM_SURE = 'Sei sicuro di voler rimuovere questo utente?'
    USER_RM_YES = em('Sì :skull:')
    USER_RM_NO = em('No :no_good:')

    # Generic sentences
    USER_EXIT = em('Esci :back:')
    ERROR_EXIT = em('Si è verificato un errore :scream:\n' + \
            'Ti riporto al menù principale...')

    # States for the conversation
    ACTIONS = 0
    ADD = 1
    ADD_NAME = 2
    REMOVE = 3
    REMOVE_SURE = 4

    ## Initializes the ConversationHandler with the states
    #
    # We take as parameter permissions and sendKeyboard because we need to have
    # a permissions object that is shared with the main bot class and we need
    # the function to send the keyboard when we exit from this state.
    def __init__(self, permissions, sendKeyboard):
        self.permissions = permissions
        self.sendKeyboard = sendKeyboard

        buttonRegex = '^({}|{})$'.format(
            re.escape(self.USER_ADD.strip()),
            re.escape(self.USER_RM.strip()),
        )

        rmSure = '^{}$'.format(re.escape(self.USER_RM_YES.strip()))
        rmNoSure = '^{}$'.format(re.escape(self.USER_RM_NO.strip()))

        ConversationHandler.__init__(self,
            entry_points=[CommandHandler('utenti', self.users,
                    pass_chat_data=True)],
            states={
                self.ACTIONS: [RegexHandler(buttonRegex, self.actions,
                        pass_chat_data=True),
                        RegexHandler('^/r([0-9]+)$', self.remove,
                        pass_chat_data=True)],
                self.ADD: [RegexHandler('^/a([0-9]+)$', self.add,
                        pass_chat_data=True)],
                self.ADD_NAME: [MessageHandler(Filters.text, self.addName,
                        pass_chat_data=True)],
                self.REMOVE: [MessageHandler(Filters.text, self.remove,
                        pass_chat_data=True)],
                self.REMOVE_SURE: [RegexHandler(rmSure, self.removeSure,
                        pass_chat_data=True), RegexHandler(rmNoSure, self.exit)]
            },
            fallbacks=[
                RegexHandler('^'+re.escape(self.USER_EXIT)+'$', self.exit),
                CommandHandler('annulla', self.exit)
            ],
        )

    ## Handle the command that will initialize the conversation
    def users(self, bot, update, chat_data):
        if self._notAdmin(update):
            return ConversationHandler.END

        reply = self.USER_CURRENT

        l = self.permissions.getUserList()
        chat_data['currentUsers'] = l
        i = 1
        for user in l:
            reply += '\n/r{} {}'.format(i, user[1])
            i += 1

        keyboard = ReplyKeyboardMarkup([[self.USER_ADD, self.USER_RM,
                self.USER_EXIT]], resize_keyboard=True)
        update.message.reply_text(reply, reply_markup=keyboard)

        return self.ACTIONS

    ## Handle the choice of action to perform
    def actions(self, bot, update, chat_data):
        if update.message.text == self.USER_ADD:
            l = self.permissions.getUnknown()
            if l:
                chat_data['availableUsers'] = l
                return self._addList(update, l)
            else:
                return self._end(update, self.USER_NO_UNKNOWN)

        elif update.message.text == self.USER_RM:
            update.message.reply_text(self.USER_RM_QUERY)
            return self.REMOVE

    ## Send the list of unknown users
    def _addList(self, update, l):
        reply = self.USER_WHO
        i = 1
        for user in l:
            name = user.first_name
            if user.last_name:
                name += ' ' + user.last_name
            if user.username:
                name += ' (@{})'.format(user.username)
            reply += '\n/a{} {}'.format(i, name)
            i += 1

        update.message.reply_text(reply)

        return self.ADD

    ## Handle the data of the user to add
    def add(self, bot, update, chat_data):
        try:
            idx = int(update.message.text[2:])
            user = chat_data['availableUsers'][idx-1]
        except:
           return self._end(update, self.ERROR_EXIT)

        chat_data['toAdd'] = user

        keyboard =  [[]]

        name = user.first_name
        if user.last_name:
            name += ' ' + user.last_name
        keyboard[0].append(name)

        if user.username:
            keyboard[0].append(user.username)

        update.message.reply_text(self.USER_ADD_NAME, reply_markup=\
                ReplyKeyboardMarkup(keyboard, resize_keyboard=True))
        return self.ADD_NAME

    ## Handling the name reception during the addition
    def addName(self, bot, update, chat_data):
        # Before doing actually anything, check again if one is admin
        if self._notAdmin(update):
            return ConversationHandler.END

        name = update.message.text
        if name[0] == '/':
            update.message.reply_text(self.USER_INVALID_NAME)
            return self.ADD_NAME
        self.permissions.addUser(chat_data['toAdd'], name)

        self.sendKeyboard(None, self.WELCOME_NEW, bot=bot,
                chatId=chat_data['toAdd'].id)

        return self._end(update, random.choice(self.SUCCESS))

    ## Get the user during the removal phase
    def remove(self, bot, update, chat_data):
        text = update.message.text
        user = None

        try:
            if text[0:2] == '/r':
                idx = int(text[2:])
                user = chat_data['currentUsers'][idx-1]
            else:
                for u in chat_data['currentUsers']:
                    if u[1] == text:
                        user = u

            if not user or self.permissions.isSuperAdmin(user[0]):
                raise ValueError()
        except:
            return self._end(update, self.ERROR_EXIT)

        kb = ReplyKeyboardMarkup([[self.USER_RM_YES, self.USER_RM_NO]],
                resize_keyboard=True)
        update.message.reply_text(self.USER_RM_SURE, reply_markup=kb)

        chat_data['toRemove'] = user
        return self.REMOVE_SURE

    ## Remove the user when the admin is sure to do it
    def removeSure(self, bot, update, chat_data):
        # Before doing actually anything, check again if one is admin
        if self._notAdmin(update):
            return ConversationHandler.END

        self.permissions.deleteUser(chat_data['toRemove'][0])
        return self._end(update, random.choice(self.SUCCESS))

    ## Exits from the add user conversation
    def exit(self, bot, update):
        return self._end(update, self.CHOOSE_ACTION)

    ## Check if a user is not an administrator
    def _notAdmin(self, update):
        if self.permissions.isAdmin(update.effective_user):
            return False
        else:
            update.message.reply_text(random.choice(self.UNAUTHORIZED))
            return True

    ## End the conversation and sends the normal keyboard
    def _end(self, update, text):
        self.sendKeyboard(update, text)
        return ConversationHandler.END
